@@ -19,9 +19,9 @@
 
 """Custom script for Gecko toolkit.  NOT WORKING WELL AT THE MOMENT."""
 
-__id__        = "$Id: Gecko.py,v 1.41 2006/11/29 00:36:29 wwalker Exp $"
-__version__   = "$Revision: 1.41 $"
-__date__      = "$Date: 2006/11/29 00:36:29 $"
+__id__        = "$Id: Gecko.py,v 1.49 2006/12/15 21:48:44 wwalker Exp $"
+__version__   = "$Revision: 1.49 $"
+__date__      = "$Date: 2006/12/15 21:48:44 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -96,11 +96,9 @@ OBJECT_ROLES = [rolenames.ROLE_CHECK_BOX,
                 rolenames.ROLE_RADIO_MENU,
                 rolenames.ROLE_ROW_HEADER,
                 rolenames.ROLE_SECTION,
-                rolenames.ROLE_SEPARATOR,
                 rolenames.ROLE_SLIDER,
                 rolenames.ROLE_SPIN_BUTTON,
                 rolenames.ROLE_STATUSBAR,
-                rolenames.ROLE_TABLE_CELL,
                 rolenames.ROLE_TABLE_COLUMN_HEADER,
                 rolenames.ROLE_TABLE_ROW_HEADER,
                 rolenames.ROLE_TEAR_OFF_MENU_ITEM,
@@ -219,11 +217,11 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             regions.append(braille.Region(label + " "))
             focusedRegionIndex = 1
 
-        if menu.name:
+        if menu and menu.name:
             regions.append(braille.Region(menu.name))
         else:
             name = obj.name
-            if menu.state.count(atspi.Accessibility.STATE_VISIBLE):
+            if menu and menu.state.count(atspi.Accessibility.STATE_VISIBLE):
                 selection = menu.selection
                 item = selection.getSelectedChild(0)
                 name = item.name
@@ -261,9 +259,35 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
     def __init__(self, script):
         speechgenerator.SpeechGenerator.__init__(self, script)
-        self.speechGenerators[rolenames.ROLE_ENTRY]        = \
+        self.speechGenerators[rolenames.ROLE_DOCUMENT_FRAME] = \
+             self._getSpeechForDocumentFrame
+        self.speechGenerators[rolenames.ROLE_ENTRY]          = \
              self._getSpeechForText
 
+    def _getSpeechForDocumentFrame(self, obj, already_focused):
+        """Gets the speech for a document frame.
+
+        Arguments:
+        - obj: an Accessible
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+        name = obj.name
+        if name and len(name):
+            utterances.append(name)
+
+        utterances.extend(self._getSpeechForObjectRole(obj))
+
+        self._debugGenerator("Gecko._getSpeechForDocumentFrame",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
     def _getSpeechForText(self, obj, already_focused):
         """Gets the speech for an autocomplete box.
 
@@ -332,11 +356,11 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         if not already_focused:
             utterances.extend(self._getSpeechForObjectRole(obj))
 
-        if menu.name:
+        if menu and menu.name:
             utterances.append(menu.name)
         else:
             name = obj.name
-            if menu.state.count(atspi.Accessibility.STATE_VISIBLE):
+            if menu and menu.state.count(atspi.Accessibility.STATE_VISIBLE):
                 selection = menu.selection
                 item = selection.getSelectedChild(0)
                 name = item.name
@@ -696,7 +720,7 @@ class Script(default.Script):
         if user_bindings:
             handler = user_bindings.getInputHandler(keyboardEvent)
             if handler and handler._function in self._caretNavigationFunctions:
-                return self.useCaretNavigationModel()
+                return self.useCaretNavigationModel(keyboardEvent)
             elif handler \
                 and handler._function in self._structuralNavigationFunctions:
                 return self.useStructuralNavigationModel()
@@ -705,7 +729,7 @@ class Script(default.Script):
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler and handler._function in self._caretNavigationFunctions:
-                return self.useCaretNavigationModel()
+                return self.useCaretNavigationModel(keyboardEvent)
             elif handler \
                 and handler._function in self._structuralNavigationFunctions:
                 return self.useStructuralNavigationModel()
@@ -723,6 +747,15 @@ class Script(default.Script):
         accessible text specialization, the characterOffset value
         is meaningless (and typically -1)."""
 
+        # We need to handle HTML content differently because we do our
+        # own navigation and we also handle the EMBEDDED_OBJECT_CHARACTER.
+        # But...if we're not in HTML content, we'll defer to the default
+        # script.
+        #
+        if not self.inDocumentContent():
+            default.Script.onCaretMoved(self, event)
+            return
+
         self.caretContext = self.getFirstCaretContext(\
             event.source,
             event.source.text.caretOffset)
@@ -765,7 +798,6 @@ class Script(default.Script):
         # on the context.
         #
         if (event.source.role == rolenames.ROLE_FRAME) \
-           or (event.source.role == rolenames.ROLE_DOCUMENT_FRAME) \
            or (not len(event.source.role)):
             return
 
@@ -806,10 +838,20 @@ class Script(default.Script):
 
         # Autocomplete widgets are a complex beast as well.  When they
         # get focus, their child (which is an entry) really has focus.
+        # Their child also issues a focus: event, so we just ignore
+        # the autocomplete focus: event.
         #
         if event.source.role == rolenames.ROLE_AUTOCOMPLETE:
-            entry = self.getAutocompleteEntry(event.source)
-            orca.setLocusOfFocus(event, entry)
+            # [[[WDW - we used to force the locus of focus to the
+            # entry.  The idea was that even if the entry issued
+            # a focus: event, the locus of focus would not change.
+            # The problem, however, is that Gecko often gives us
+            # a *different* child whenever we ask for the entry.
+            # So...we end up just depending upon the entry to issue
+            # focus: events.]]]
+            #
+            #entry = self.getAutocompleteEntry(event.source)
+            #orca.setLocusOfFocus(event, entry)
             return
 
         default.Script.onFocus(self, event)
@@ -1111,27 +1153,39 @@ class Script(default.Script):
                 obj = obj.parent
         return False
 
-    def useCaretNavigationModel(self):
+    def useCaretNavigationModel(self, keyboardEvent):
         """Returns True if we should do our own caret navigation.
-        [[[TODO: WDW - this should return False if we're in something
-        like an entry area or a list because we want their keyboard
-        navigation stuff to work.]]]
         """
 
         if not controlCaretNavigation:
             return False
 
-        letThemDoItRoles = [rolenames.ROLE_ENTRY]
-        obj = orca_state.locusOfFocus
-        while obj:
-            if obj.role == rolenames.ROLE_DOCUMENT_FRAME:
-                return True
-            elif obj.role in letThemDoItRoles:
-                return False
-            else:
-                obj = obj.parent
+        if not self.inDocumentContent():
+            return False
 
-        return False
+        weHandleIt = False
+        obj = orca_state.locusOfFocus
+        if obj.role == rolenames.ROLE_ENTRY:
+            text        = obj.text
+            length      = text.characterCount
+            caretOffset = text.caretOffset
+            singleLine  = obj.state.count(
+                atspi.Accessibility.STATE_SINGLE_LINE)
+            if length == 0:
+                weHandleIt = True
+            elif caretOffset <= 0:
+                weHandleIt = keyboardEvent.event_string \
+                             in ["Up", "Left"]
+            elif caretOffset >= length - 1:
+                weHandleIt = keyboardEvent.event_string \
+                             in ["Down", "Right"]
+            else:
+                weHandleIt = False
+
+            if singleLine and not weHandleIt:
+                weHandleIt = keyboardEvent.event_string in ["Up", "Down"]
+
+        return weHandleIt
 
     def useStructuralNavigationModel(self):
         """Returns True if we should do our own structural navigation.
@@ -1988,12 +2042,6 @@ class Script(default.Script):
 
         self.caretContext = [obj, characterOffset]
 
-        # We'd like the thing to have focus if it can take focus.
-        #
-        focusGrabbed = obj.component.grabFocus()
-        if not focusGrabbed:
-            print "FOCUS NOT GRABBED", obj.role, characterOffset
-
         # If there is a character there, we'd like to position the
         # caret the right spot.  [[[TODO: WDW - numbered lists are
         # whacked in that setting the caret offset somewhere in
@@ -2008,18 +2056,28 @@ class Script(default.Script):
                 caretSet = False
             if not caretSet:
                 print "CARET NOT SET", obj.role, characterOffset
+            else:
+                #print "CARET SET", obj.role, characterOffset
+                pass
+
+        # We'd like the thing to have focus if it can take focus.
+        #
+        focusGrabbed = obj.component.grabFocus()
+        if not focusGrabbed:
+            print "FOCUS NOT GRABBED", obj.role, characterOffset
+        else:
+            #print "FOCUS GRABBED", obj.role, characterOffset
+            pass
 
     def goNextCharacter(self, inputEvent):
         """Positions the caret offset to the next character or object
         in the document window.
         """
-
         [obj, characterOffset] = self.getCaretContext()
         while obj:
             [obj, characterOffset] = self.getNextCaretInOrder(obj,
                                                               characterOffset)
             if obj and obj.state.count(atspi.Accessibility.STATE_SHOWING):
-                self.caretContext = [obj, characterOffset]
                 break
 
         if obj:
@@ -2039,7 +2097,6 @@ class Script(default.Script):
             [obj, characterOffset] = self.getPreviousCaretInOrder(
                 obj, characterOffset)
             if obj and obj.state.count(atspi.Accessibility.STATE_SHOWING):
-                self.caretContext = [obj, characterOffset]
                 break
 
         if obj:
