@@ -25,10 +25,10 @@ ACSS definitions into engine-specific markup codes.
 
 """
 
-__id__ = "$Id: acss.py,v 1.5 2006/10/06 07:59:03 richb Exp $"
-__author__ = "$Author: richb $"
-__version__ = "$Revision: 1.5 $"
-__date__ = "$Date: 2006/10/06 07:59:03 $"
+__id__ = "$Id: acss.py,v 1.6 2006/11/09 18:43:48 wwalker Exp $"
+__author__ = "T. V. Raman"
+__version__ = "$Revision: 1.6 $"
+__date__ = "$Date: 2006/11/09 18:43:48 $"
 __copyright__ = "Copyright (c) 2005 Google Inc."
 __license__ = "LGPL"
 
