@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py,v 1.50 2006/10/26 16:58:17 richb Exp $"
-__version__   = "$Revision: 1.50 $"
-__date__      = "$Date: 2006/10/26 16:58:17 $"
+__id__        = "$Id: atspi.py,v 1.51 2006/12/12 03:40:49 lmonsanto Exp $"
+__version__   = "$Revision: 1.51 $"
+__date__      = "$Date: 2006/12/12 03:40:49 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1371,7 +1371,8 @@ class Accessible:
                 newChild.app = self.app
 
         if not newChild:
-            raise Exception("Invalid child at index: %d" % index)
+            debug.println(debug.LEVEL_FINEST,
+                          "Child at index %d is not an Accessible" % index)
 
         return newChild
 
