@@ -25,9 +25,9 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py,v 1.66 2006/11/06 16:41:03 wwalker Exp $"
-__version__   = "$Revision: 1.66 $"
-__date__      = "$Date: 2006/11/06 16:41:03 $"
+__id__        = "$Id: braille.py,v 1.73 2006/12/17 23:45:47 wwalker Exp $"
+__version__   = "$Revision: 1.73 $"
+__date__      = "$Date: 2006/12/17 23:45:47 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -50,6 +50,7 @@ try:
 except:
     import brl
     useBrlAPIBindings = False
+    brlAPIRunning = False
 
 try:
     # This can fail due to gtk not being available.  We want to
@@ -680,7 +681,14 @@ def refresh(panToCursor=True, targetCursorCell=0):
     substring = string[startPos:endPos].encode("UTF-8")
     if useBrlAPIBindings:
         if brlAPIRunning:
-            writeStruct = brlapi.Write()
+            try:
+                # The name after (and including) BrlTTY v3.8 revision 2810
+                #
+                writeStruct = brlapi.WriteStruct()
+            except:
+                # The name before BrlTTY v3.8 revision 2810
+                #
+                writeStruct = brlapi.Write()
             writeStruct.text = substring
             writeStruct.regionBegin = 1
             writeStruct.regionSize = len(substring.decode("UTF-8"))
@@ -689,8 +697,23 @@ def refresh(panToCursor=True, targetCursorCell=0):
                 writeStruct.regionSize += 1
             writeStruct.cursor = cursorCell
             writeStruct.charset = "UTF-8"
+
+            # [[[WDW - if you want to muck around with the dots on the
+            # display to do things such as add underlines, you can use
+            # the attrOr field of the write structure to do so.  The
+            # attrOr field is a string whose length must be the same
+            # length as the display and whose dots will end up showing
+            # up on the display.  Each character represents a bitfield
+            # where each bit corresponds to a dot (i.e., bit 0 = dot 1,
+            # bit 1 = dot 2, and so on).  Here's an example that underlines
+            # all the text.]]]
+            #
+            #myUnderline = ""
+            #for i in range(0, _displaySize[0]):
+            #    myUnderline += '\xc0'
+            #writeStruct.attrOr = myUnderline
+
             brlAPI.write(writeStruct)
-            #brlAPI.writeText(substring, cursorCell)
     else:
         brl.writeText(cursorCell, substring)
 
@@ -893,6 +916,38 @@ def _brlAPIKeyReader():
                 _processBrailleEvent(keyCode)
         return brlAPIRunning
 
+def setupKeyRanges(keys):
+    """Hacky method to tell BrlTTY what to send and not send us via
+    the readKey method.  This only works with BrlTTY v3.8 and better.
+
+    Arguments:
+    -keys: a list of BrlAPI commands.
+    """
+    if not brlAPIRunning:
+        return
+
+    # First, start by ignoring everything.
+    #
+    everything=[0,
+                brlapi.KEY_FLAGS_MASK \
+                | brlapi.KEY_TYPE_MASK \
+                | brlapi.KEY_CODE_MASK]
+    brlAPI.ignoreKeyRange(everything)
+
+    # Next, enable cursor routing keys.
+    #
+    cursorRoutingKeys = [brlapi.KEY_CMD_ROUTE,
+                         brlapi.KEY_CMD_ROUTE | brlapi.KEY_CMD_ARG_MASK]
+    brlAPI.acceptKeyRange(cursorRoutingKeys)
+
+    # Finally, enable the commands we care about.
+    #
+    keySet = []
+    for key in keys:
+        keySet.append(brlapi.KEY_TYPE_CMD | key)
+    if len(keySet):
+        brlAPI.acceptKeySet(keySet)
+
 def init(callback=None, tty=7):
     """Initializes the braille module, connecting to the BrlTTY driver.
 
@@ -918,10 +973,25 @@ def init(callback=None, tty=7):
             gobject.threads_init()
             global brlAPI
             global brlAPIRunning
-            brlAPI = brlapi.Bridge()
-            brlAPI.getTty(tty)
-            brlAPIRunning = True
-            gobject.idle_add(_brlAPIKeyReader)
+            brlAPI = brlapi.Connection()
+
+            try:
+                import os
+                windowPath = os.environ["WINDOWPATH"]
+                brlAPI.enterTtyModeWithPath()
+                brlAPIRunning = True
+                gobject.idle_add(_brlAPIKeyReader)
+                debug.println(\
+                    debug.LEVEL_CONFIGURATION,
+                    "Braille module has been initialized using WINDOWPATH=" \
+                    + "%s" % windowPath)
+            except:
+                brlAPI.enterTtyMode(tty)
+                brlAPIRunning = True
+                gobject.idle_add(_brlAPIKeyReader)
+                debug.println(\
+                    debug.LEVEL_CONFIGURATION,
+                    "Braille module has been initialized using tty=%d" % tty)
         except:
             debug.printException(debug.LEVEL_FINEST)
             return False
@@ -940,7 +1010,7 @@ def init(callback=None, tty=7):
     #
     #_displaySize = (brl.getDisplayWidth(), brl.getDisplayHeight())
     if useBrlAPIBindings:
-        (x, y) = brlAPI.displaysize
+        (x, y) = brlAPI.displaySize
         _displaySize = [x, 1]
     else:
         _displaySize = [brl.getDisplayWidth(), 1]
@@ -970,7 +1040,7 @@ def shutdown():
     if useBrlAPIBindings:
         if brlAPIRunning:
             brlAPIRunning = False
-            brlAPI.leaveTty()
+            brlAPI.leaveTtyMode()
     else:
         brl.shutdown()
 
