@@ -22,9 +22,9 @@ level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py,v 1.30 2006/10/13 23:39:04 wwalker Exp $"
-__version__   = "$Revision: 1.30 $"
-__date__      = "$Date: 2006/10/13 23:39:04 $"
+__id__        = "$Id: debug.py,v 1.31 2006/12/12 03:40:50 lmonsanto Exp $"
+__version__   = "$Revision: 1.31 $"
+__date__      = "$Date: 2006/12/12 03:40:50 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -205,5 +205,5 @@ def printDetails(level, indent, accessible, includeApp=True):
     - includeApp: if True, include information about the app
     """
 
-    if accessible:
+    if level >= debugLevel and accessible:
         println(level, accessible.toString(indent, includeApp))
