@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py,v 1.48 2006/11/07 19:14:01 lmonsanto Exp $"
-__version__   = "$Revision: 1.48 $"
-__date__      = "$Date: 2006/11/07 19:14:01 $"
+__id__        = "$Id: flat_review.py,v 1.51 2006/12/06 19:12:55 wwalker Exp $"
+__version__   = "$Revision: 1.51 $"
+__date__      = "$Date: 2006/12/06 19:12:55 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -434,16 +434,22 @@ class Context:
             zoneIndex   = currentZoneIndex
             caretOffset = zone.accessible.text.caretOffset
             foundZoneWithCaret = False
+            checkForEOF = False
             while lineIndex < len(self.lines):
                 line = self.lines[lineIndex]
                 while zoneIndex < len(line.zones):
                     zone = line.zones[zoneIndex]
                     if zone.accessible == accessible:
-                        if (caretOffset >= zone.startOffset) \
-                               and (caretOffset \
+                        if (caretOffset >= zone.startOffset):
+                            if (caretOffset \
                                     < (zone.startOffset + zone.length)):
-                            foundZoneWithCaret = True
-                            break
+                                foundZoneWithCaret = True
+                                break
+                            elif (caretOffset \
+                                    == (zone.startOffset + zone.length)):
+                                checkForEOF = True
+                                lineToCheck = lineIndex
+                                zoneToCheck = zoneIndex
                     zoneIndex += 1
                 if foundZoneWithCaret:
                     currentLineIndex = lineIndex
@@ -463,6 +469,16 @@ class Context:
                 else:
                     zoneIndex = 0
                     lineIndex += 1
+            atEOF = not foundZoneWithCaret and checkForEOF
+            if atEOF:
+                line = self.lines[lineToCheck]
+                zone = line.zones[zoneToCheck]
+                currentLineIndex = lineToCheck
+                currentZoneIndex = zoneToCheck
+                if caretOffset:
+                    currentWordIndex = len(zone.words) - 1
+                    currentCharIndex = \
+                          zone.words[currentWordIndex].length - 1
 
         self.lineIndex = currentLineIndex
         self.zoneIndex = currentZoneIndex
