@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py,v 1.86 2006/09/12 11:19:56 wwalker Exp $"
-__version__   = "$Revision: 1.86 $"
-__date__      = "$Date: 2006/09/12 11:19:56 $"
+__id__        = "$Id: focus_tracking_presenter.py,v 1.95 2006/12/13 22:35:06 wwalker Exp $"
+__version__   = "$Revision: 1.95 $"
+__date__      = "$Date: 2006/12/13 22:35:06 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,6 +37,7 @@ import debug
 import input_event
 import orca_state
 import presentation_manager
+import rolenames
 import settings
 import speech
 import util
@@ -67,6 +68,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         self.registry        = atspi.Registry()
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._eventQueue     = Queue.Queue(0)
         self._gidleId        = 0
         self._gidleLock      = threading.Lock()
@@ -293,6 +296,44 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         except:
             debug.printException(debug.LEVEL_FINEST)
 
+    ########################################################################
+    #                                                                      #
+    # METHODS FOR KEEPING TRACK OF APPLICATION SETTINGS.                   #
+    #                                                                      #
+    ########################################################################
+
+    def loadAppSettings(self, app):
+        """Load the users application specific settings for an app.
+
+        Arguments:
+        - app: the Python app
+        """
+
+        settingsPackages = settings.settingsPackages
+        moduleName = settings.getScriptModuleName(app)
+        module = None
+
+        if moduleName and len(moduleName):
+            for package in settingsPackages:
+                if len(package):
+                    name = package + "." + moduleName
+                else:
+                    name = moduleName
+                try:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "Looking for settings at %s.py..." % name)
+                    if self. _knownAppSettings.has_key(name):
+                        reload(self._knownAppSettings[name])
+                    else:
+                        self._knownAppSettings[name] = \
+                            __import__(name, globals(), locals(), [''])
+                    debug.println(debug.LEVEL_FINEST,
+                                  "...found %s.py" % name)
+                    break
+                except ImportError:
+                    debug.println(debug.LEVEL_FINEST,
+                                  "...could not find %s.py" % name)
+
     ########################################################################
     #                                                                      #
     # METHODS FOR PRE-PROCESSING AND MASSAGING AT-SPI OBJECT EVENTS        #
@@ -336,6 +377,21 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         debug.printObjectEvent(debug.LEVEL_FINEST, event)
 
+        # [[[TODO: WDW - HACK to prevent gnome-panel from killing
+        # itself.  It seems to do so after it issues some tool tip
+        # events and Orca attempts to process them.  We're not really
+        # doing anything with tool tips right now, so we just ignore
+        # them.  Note that this is just a bandaid to the problem.  We
+        # should do something better.  Please refer to bug 368626
+        # http://bugzilla.gnome.org/show_bug.cgi?id=368626 to follow
+        # this problem.]]]
+        #
+        try:
+            if event.source.role == rolenames.ROLE_TOOL_TIP:
+                return
+        except:
+            pass
+
         # Reclaim (delete) any scripts when desktop children go away.
         # The idea here is that a desktop child is an app. We also
         # generally do not like object:children-changed:remove events,
@@ -399,18 +455,52 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         oldLocusOfFocus = orca_state.locusOfFocus
         while retryCount <= settings.commFailureAttemptLimit:
             try:
-                if event.type == "window:activate":
-                    # We'll let someone else decide if it's important
-                    # to stop speech or not.
-                    #speech.stop()
-                    orca_state.activeScript = self._getScript(event.source.app)
-                    debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
-                                  + orca_state.activeScript.name)
-                s = self._getScript(event.source.app)
-                s.processObjectEvent(event)
-                if retryCount:
-                    debug.println(debug.LEVEL_WARNING,
-                                  "  SUCCEEDED AFTER %d TRIES" % retryCount)
+                if not event.source.state.count( \
+                                      atspi.Accessibility.STATE_ICONIFIED):
+
+                    # [[[TODO: WDW - HACK we look for frame that get
+                    # focus: as a means to detect active scripts
+                    # because yelp does this.  Actually, yelp is a bit
+                    # odd in that it calls itself 'yelp' then changes
+                    # its application name and id to the Gecko toolkit
+                    # in use, and then issues a focus: event on the
+                    # main window, which is a frame.]]]
+                    #
+                    if (event.type == "window:activate") \
+                       or ((event.type == "focus:") \
+                           and (event.source.role == rolenames.ROLE_FRAME)):
+
+                        # If old ("factory") settings don't exist yet, save
+                        # a set, else restore the old application settings.
+                        #
+                        if not self._oldAppSettings:
+                            self._oldAppSettings = util.saveOldAppSettings()
+                        else:
+                            util.restoreOldAppSettings(self._oldAppSettings)
+
+                        # We'll let someone else decide if it's important
+                        # to stop speech or not.
+                        #speech.stop()
+                        orca_state.activeScript = \
+                            self._getScript(event.source.app)
+                        debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
+                                      + orca_state.activeScript.name)
+
+                        # Load in the application specific settings for the
+                        # app for this event (if found).
+                        #
+                        appSettings = self.loadAppSettings(event.source.app)
+
+                        # Tell BrlTTY which commands we care about.
+                        #
+                        braille.setupKeyRanges(\
+                            orca_state.activeScript.brailleBindings.keys())
+
+                    s = self._getScript(event.source.app)
+                    s.processObjectEvent(event)
+                    if retryCount:
+                        debug.println(debug.LEVEL_WARNING,
+                                      "  SUCCEEDED AFTER %d TRIES" % retryCount)
                 break
             except CORBA.COMM_FAILURE:
                 debug.printException(debug.LEVEL_WARNING)
@@ -736,10 +826,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._listenerCounts = {}
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._defaultScript  = None
 
         orca_state.activeScript = self._getScript(None)
 
+        # Tell BrlTTY which commands we care about.
+        #
+        braille.setupKeyRanges(orca_state.activeScript.brailleBindings.keys())
+
         self._registerEventListener("window:activate")
         self._registerEventListener("window:deactivate")
         self._registerEventListener("object:children-changed:remove")
@@ -765,6 +861,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                                   eventType)
         self._listenerCounts = {}
         self._knownScripts   = {}
+        self._knownAppSettings = {}
+        self._oldAppSettings = None
         self._defaultScript  = None
 
         orca_state.activeScript = None
