@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.57 2006/10/27 16:25:43 mpeders Exp $"
-__version__   = "$Revision: 1.57 $"
-__date__      = "$Date: 2006/10/27 16:25:43 $"
+__id__        = "$Id: gnomespeechfactory.py,v 1.58 2006/12/06 17:33:21 wwalker Exp $"
+__version__   = "$Revision: 1.58 $"
+__date__      = "$Date: 2006/12/06 17:33:21 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -629,7 +629,7 @@ class SpeechServer(speechserver.SpeechServer):
                 removeNewLines = False
 
         if removeNewLines:
-            oldText = oldText.replace("\n", _(""), 1)
+            oldText = oldText.replace("\n", "", 1)
 
         style = settings.verbalizePunctuationStyle
         newText = ''
