@@ -20,9 +20,9 @@
 """Uses speech prompts and a command line interface to set Orca
 user preferences."""
 
-__id__        = "$Id: orca_console_prefs.py,v 1.9 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.9 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: orca_console_prefs.py,v 1.14 2006/11/17 20:33:57 wwalker Exp $"
+__version__   = "$Revision: 1.14 $"
+__date__      = "$Date: 2006/11/17 20:33:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -31,6 +31,16 @@ import settings
 import speech
 import speechserver
 import orca_prefs
+import platform
+import time
+
+desktopRunning = False
+try:
+    import gtk
+    if gtk.gdk.display_get_default():
+        desktopRunning = True
+except:
+    pass
 
 from orca_i18n import _  # for gettext support
 
@@ -191,7 +201,7 @@ def setupSpeech(prefsDict):
     defaultACSS[acss.ACSS.GAIN] = 9
     defaultACSS[acss.ACSS.AVERAGE_PITCH] = 5
     uppercaseACSS = acss.ACSS({acss.ACSS.AVERAGE_PITCH : 6})
-    hyperlinkACSS = acss.ACSS({acss.ACSS.AVERAGE_PITCH : 2})
+    hyperlinkACSS = acss.ACSS({})
 
     voices = {
         settings.DEFAULT_VOICE   : defaultACSS,
@@ -286,8 +296,42 @@ def setupSpeech(prefsDict):
         prefsDict["enableFunctionKeys"]  = False
         prefsDict["enableActionKeys"]    = False
 
+    sayAndPrint(_("Select desired keyboard layout."),
+                False,
+                False,
+                speechServerChoice,
+                speechVoiceChoice)
+    i = 1
+    choices = {}
+    sayAndPrint(_("1. Desktop"))
+    sayAndPrint(_("2. Laptop"))
+    choice = int(sayAndPrint(_("Enter choice: "), False, True))
+    if choice == 2:
+        prefsDict["keyboardLayout"] = settings.GENERAL_KEYBOARD_LAYOUT_LAPTOP
+        prefsDict["orcaModifierKeys"] = settings.LAPTOP_MODIFIER_KEYS
+    else:
+        prefsDict["keyboardLayout"] = settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP
+        prefsDict["orcaModifierKeys"] = settings.DESKTOP_MODIFIER_KEYS
+    if (choice <= 0) or (choice >= 3):
+        sayAndPrint(_("Invalid choice. Selecting desktop keyboard layout.\n"))
+
     return True
 
+def logoutUser():
+    """Automatically log the user out of the GNOME desktop."""
+
+    import gnome
+    import gnome.ui
+
+    program = gnome.init(platform.package, platform.version)
+    client = gnome.ui.master_client()
+
+    client.request_save(gnome.ui.SAVE_GLOBAL,  # Save style
+                        True,                  # Shutdown
+                        gnome.ui.INTERACT_ANY, # Allow user interaction
+                        False,                 # Fast
+                        True)                  # All apps save state
+
 def showPreferencesUI():
     """Uses the console to query the user for Orca preferences."""
 
@@ -327,6 +371,27 @@ def showPreferencesUI():
                     speechServerChoice,
                     speechVoiceChoice)
 
+        if desktopRunning:
+            answer = sayAndPrint(_("Do you want to logout now?  Enter y or n: "),
+                                 False,
+                                 True,
+                                 speechServerChoice,
+                                 speechVoiceChoice)
+            if answer[0:1] == 'Y' or answer[0:1] == 'y':
+                sayAndPrint(_("Setup complete. Logging out now."),
+                            False,
+                            False,
+                            speechServerChoice,
+                            speechVoiceChoice)
+                time.sleep(2)
+
+                import bonobo
+                import gobject
+
+                gobject.threads_init()
+                gobject.idle_add(logoutUser)
+                bonobo.main()
+
     answer = sayAndPrint(_("Setup complete.  Press Return to continue."),
                          not logoutNeeded,
                          True,
