@@ -35,3 +35,11 @@ _ = gettext.gettext
 localedir = os.path.join ("/usr", "share", "locale")
 gettext.bindtextdomain ("orca", localedir)
 gettext.textdomain("orca")
+
+try:
+    import gtk.glade
+
+    gtk.glade.bindtextdomain ("orca", localedir)
+    gtk.glade.textdomain("orca")
+except:
+    debug.printException(debug.LEVEL_FINEST)
