@@ -20,9 +20,9 @@
 """Holds state that is shared among many modules.
 """
 
-__id__        = "$Id: orca_state.py 1291 2006-08-08 15:04:41Z richb $"
-__version__   = "$Revision: 1291 $"
-__date__      = "$Date: 2006-08-08 11:04:41 -0400 (Tue, 08 Aug 2006) $"
+__id__        = "$Id: orca_state.py,v 1.7 2006/12/13 01:21:31 wwalker Exp $"
+__version__   = "$Revision: 1.7 $"
+__date__      = "$Date: 2006/12/13 01:21:31 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -38,8 +38,22 @@ locusOfFocus = None
 #
 activeWindow = None
 
+# The currently active script.
+#
+activeScript = None
+
+# The "click" count. Used to determine if the user has double or triple
+# "clicked" a key.
+#
+clickCount = 0
+
+# Used to capture keys to redefine key bindings by the user.
+#
+capturingKeys   = False
+lastCapturedKey = None
+
 # The InputEvent instance representing the last input event.  This is
-# set each time a keyboard or braille event is received.
+# set each time a mouse, keyboard or braille event is received.
 #
 lastInputEvent = None
 
@@ -56,3 +70,11 @@ lastKeyEchoTime = None
 #
 noFocusTimestamp = 0.0
 
+# The last word spoken.
+#
+lastWord = ""
+
+# The last searchQuery
+#
+searchQuery = None
+
