@@ -20,9 +20,9 @@
 """Exposes a dictionary, pronunciation_dict, that maps words to what
 they sound like."""
 
-__id__        = "$Id: pronunciation_dict.py,v 1.2 2006/10/23 17:16:30 richb Exp $"
-__version__   = "$Revision: 1.2 $"
-__date__      = "$Date: 2006/10/23 17:16:30 $"
+__id__        = "$Id: pronunciation_dict.py,v 1.4 2006/11/22 15:56:10 richb Exp $"
+__version__   = "$Revision: 1.4 $"
+__date__      = "$Date: 2006/11/22 15:56:10 $"
 __copyright__ = "Copyright (c) 2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -36,8 +36,13 @@ from orca_i18n import _ # for gettext support
 #
 pronunciation_dict = {}
 pronunciation_dict[_("ASAP")]    = _("as soon as possible")
+pronunciation_dict[_("GHz")]     = _("gigahertz")
 pronunciation_dict[_("IMAP")]    = _("eye map")
+pronunciation_dict[_("LDAP")]    = _("ell dap")
 pronunciation_dict[_("LOL")]     = _("laughing out loud")
+pronunciation_dict[_("MHz")]     = _("megahertz")
+pronunciation_dict[_("strikethrough")] = _("strike through")
+pronunciation_dict[_("SELinux")] = _("ess ee linux")
 
 def getPronunciation(word):
     """Given a word, return a string that represents what this word
