@@ -21,9 +21,9 @@
 """Provides a methods that converts the role name of an Accessible
 object into localized strings for speech and braille."""
 
-__id__        = "$Id: rolenames.py,v 1.26 2006/07/13 13:46:48 wwalker Exp $"
-__version__   = "$Revision: 1.26 $"
-__date__      = "$Date: 2006/07/13 13:46:48 $"
+__id__        = "$Id: rolenames.py,v 1.31 2006/12/15 21:48:45 wwalker Exp $"
+__version__   = "$Revision: 1.31 $"
+__date__      = "$Date: 2006/12/15 21:48:45 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -44,6 +44,7 @@ ROLE_ALERT               = "alert"
 ROLE_ANIMATION           = "animation"
 ROLE_ARROW               = "arrow"
 ROLE_CALENDAR            = "calendar"
+ROLE_CAPTION             = "caption"
 ROLE_CANVAS              = "canvas"
 ROLE_CHECK_BOX           = "check box"
 ROLE_CHECK_MENU_ITEM     = "check menu item"
@@ -57,18 +58,24 @@ ROLE_DESKTOP_FRAME       = "desktop frame"
 ROLE_DIAL                = "dial"
 ROLE_DIALOG              = "dialog"
 ROLE_DIRECTORY_PANE      = "directory pane"
+ROLE_DOCUMENT_FRAME      = "document frame"
 ROLE_DRAWING_AREA        = "drawing area"
+ROLE_ENTRY               = "entry"
 ROLE_FILE_CHOOSER        = "file chooser"
 ROLE_FILLER              = "filler"
 ROLE_FONT_CHOOSER        = "fontchooser"
+ROLE_FORM                = "form"
 ROLE_FRAME               = "frame"
 ROLE_GLASS_PANE          = "glass pane"
+ROLE_HEADING             = "heading"
 ROLE_HTML_CONTAINER      = "html container"
 ROLE_ICON                = "icon"
 ROLE_IMAGE               = "image"
 ROLE_INTERNAL_FRAME      = "internal frame"
+ROLE_INPUT_METHOD_WINDOW = "input method window"
 ROLE_LABEL               = "label"
 ROLE_LAYERED_PANE        = "layered pane"
+ROLE_LINK                = "link"
 ROLE_LIST                = "list"
 ROLE_LIST_ITEM           = "list item"
 ROLE_MENU                = "menu"
@@ -89,6 +96,7 @@ ROLE_ROOT_PANE           = "root pane"
 ROLE_ROW_HEADER          = "row header"
 ROLE_SCROLL_BAR          = "scroll bar"
 ROLE_SCROLL_PANE         = "scroll pane"
+ROLE_SECTION             = "section"
 ROLE_SEPARATOR           = "separator"
 ROLE_SLIDER              = "slider"
 ROLE_SPLIT_PANE          = "split pane"
@@ -178,6 +186,11 @@ rolenames[ROLE_CANVAS] = Rolename(ROLE_CANVAS,
                                   _("Canvas"),
                                   _("canvas"))
 
+rolenames[ROLE_CAPTION] = Rolename(ROLE_CAPTION,
+                                   _("cptn"),
+                                   _("Caption"),
+                                   _("caption"))
+
 rolenames[ROLE_CHECK_BOX] = Rolename(ROLE_CHECK_BOX,
                                      _("chk"),
                                      _("CheckBox"),
@@ -238,6 +251,11 @@ rolenames[ROLE_DIRECTORY_PANE] = Rolename(ROLE_DIRECTORY_PANE,
                                           _("DirectoryPane"),
                                           _("directory pane"))
 
+rolenames[ROLE_DOCUMENT_FRAME] = Rolename(ROLE_DOCUMENT_FRAME,
+                                _("html"),
+                                _("HtmlPane"),
+                                _("html content"))
+
 rolenames[ROLE_DRAWING_AREA] = Rolename(ROLE_DRAWING_AREA,
                                         _("draw"),
                                         _("DrawingArea"),
@@ -258,6 +276,11 @@ rolenames[ROLE_FONT_CHOOSER] = Rolename(ROLE_FONT_CHOOSER,
                                         _("FontChooser"),
                                         _("font chooser"))
 
+rolenames[ROLE_FORM] = Rolename(ROLE_FORM,
+                                _("form"),
+                                _("Form"),
+                                _("form"))
+
 rolenames[ROLE_FRAME] = Rolename(ROLE_FRAME,
                                  _("frm"),
                                  _("Frame"),
@@ -268,6 +291,11 @@ rolenames[ROLE_GLASS_PANE] = Rolename(ROLE_GLASS_PANE,
                                       _("GlassPane"),
                                       _("glass pane"))
 
+rolenames[ROLE_HEADING] = Rolename(ROLE_HEADING,
+                                   _("hdng"),
+                                   _("Heading"),
+                                   _("heading"))
+
 rolenames[ROLE_HTML_CONTAINER] = Rolename(ROLE_HTML_CONTAINER,
                                           _("html"),
                                           _("HtmlContainer"),
@@ -298,6 +326,11 @@ rolenames[ROLE_LAYERED_PANE] = Rolename(ROLE_LAYERED_PANE,
                                         _("LayeredPane"),
                                         _("layered pane"))
 
+rolenames[ROLE_LINK] = Rolename(ROLE_LINK,
+                                _("lnk"),
+                                _("Link"),
+                                _("link"))
+
 rolenames[ROLE_LIST] = Rolename(ROLE_LIST,
                                 _("lst"),
                                 _("List"),
@@ -398,6 +431,11 @@ rolenames[ROLE_SCROLL_PANE] = Rolename(ROLE_SCROLL_PANE,
                                        _("ScrollPane"),
                                        _("scroll pane"))
 
+rolenames[ROLE_SECTION] = Rolename(ROLE_SECTION,
+                                   _("sctn"),
+                                   _("Section"),
+                                   _("section"))
+
 rolenames[ROLE_SEPARATOR] = Rolename(ROLE_SEPARATOR,
                                      _("seprtr"),
                                      _("Separator"),
@@ -458,6 +496,8 @@ rolenames[ROLE_TEXT] = Rolename(ROLE_TEXT,
                                 _("Text"),
                                 _("text"))
 
+rolenames[ROLE_ENTRY] = rolenames[ROLE_TEXT]
+
 rolenames[ROLE_TOGGLE_BUTTON] = Rolename(ROLE_TOGGLE_BUTTON,
                                          _("tglbtn"),
                                          _("ToggleButton"),
