@@ -27,9 +27,9 @@ The following script specific key sequences are supported:
   Insert-[1-9]  -  Speak and braille a previous chat room message.
 """
 
-__id__        = "$Id: gaim.py 1248 2006-07-28 17:34:03Z wwalker $"
-__version__   = "$Revision: 1248 $"
-__date__      = "$Date: 2006-07-28 13:34:03 -0400 (Fri, 28 Jul 2006) $"
+__id__        = "$Id: gaim.py,v 1.25 2006/10/19 18:16:54 richb Exp $"
+__version__   = "$Revision: 1.25 $"
+__date__      = "$Date: 2006/10/19 18:16:54 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -144,13 +144,15 @@ class Script(default.Script):
         debug.println(self.debugLevel, "gaim.setupInputEventHandlers.")
 
         default.Script.setupInputEventHandlers(self)
-        self.togglePrefixHandler = input_event.InputEventHandler(
-            Script.togglePrefix,
-            _("Toggle whether we prefix chat room messages with the name of the chat room."))
+        self.inputEventHandlers["togglePrefixHandler"] = \
+            input_event.InputEventHandler(
+                Script.togglePrefix,
+                _("Toggle whether we prefix chat room messages with the name of the chat room."))
 
-        self.readPreviousMessageHandler = input_event.InputEventHandler(
-            Script.readPreviousMessage,
-            _("Speak and braille a previous chat room message."))
+        self.inputEventHandlers["readPreviousMessageHandler"] = \
+            input_event.InputEventHandler(
+                Script.readPreviousMessage,
+                _("Speak and braille a previous chat room message."))
 
     def getKeyBindings(self):
         """Defines the key bindings for this script. Setup the default
@@ -168,7 +170,7 @@ class Script(default.Script):
                 "h",
                 1 << settings.MODIFIER_ORCA,
                 1 << settings.MODIFIER_ORCA,
-                self.togglePrefixHandler))
+                self.inputEventHandlers["togglePrefixHandler"]))
 
         messageKeys = [ "1", "2", "3", "4", "5", "6", "7", "8", "9" ]
         for i in range(0, len(messageKeys)):
@@ -177,7 +179,7 @@ class Script(default.Script):
                     messageKeys[i],
                     1 << settings.MODIFIER_ORCA,
                     1 << settings.MODIFIER_ORCA,
-                    self.readPreviousMessageHandler))
+                    self.inputEventHandlers["readPreviousMessageHandler"]))
 
         return keyBindings
 
@@ -319,8 +321,9 @@ class Script(default.Script):
             if self.flatReviewContext:
                 self.toggleFlatReviewMode()
 
-            message = event.source.text.getText(event.detail1,
-                                                event.detail1 + event.detail2)
+            message = self.getText(event.source,
+                                   event.detail1,
+                                   event.detail1 + event.detail2)
             if message and message[0] == "\n":
                 message = message[1:]
 
