@@ -21,15 +21,20 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py,v 1.91 2006/11/30 16:38:58 richb Exp $"
-__version__   = "$Revision: 1.91 $"
-__date__      = "$Date: 2006/11/30 16:38:58 $"
+__id__        = "$Id: settings.py,v 1.94 2006/12/13 01:21:31 wwalker Exp $"
+__version__   = "$Revision: 1.94 $"
+__date__      = "$Date: 2006/12/13 01:21:31 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
 import re
 import sys
+
+screenWidth = 640
+screenHeight = 480
+tty = 7
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -37,13 +42,22 @@ try:
     # the desktop is not running.
     #
     import gtk.gdk
-    screenWidth = gtk.gdk.screen_get_default().get_width()
-    screenHeight = gtk.gdk.screen_get_default().get_height()
-except:
-    # Just default to something that hopefully everybody has.
+    _display = gtk.gdk.display_get_default()
+    _screen = _display.get_default_screen()
+    _root_window = _screen.get_root_window()
+
+    # These are used for placing the magnifier zoomer.
     #
-    screenWidth = 640
-    screenHeight = 480
+    screenWidth = _screen.get_width()
+    screenHeight = _screen.get_height()
+
+    # We want to know what the tty is so we can send it to BrlAPI
+    # if possible.
+    #
+    (atom, format, data) = _root_window.property_get("XFree86_VT")
+    tty = data[0]
+except:
+    pass
 
 import debug
 from acss import ACSS
@@ -93,7 +107,7 @@ userCustomizableSettings = [
     "verbalizePunctuationStyle",
     "showMainWindow",
     "keyboardLayout",
-    "speakBlankLines",   
+    "speakBlankLines",
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -446,6 +460,11 @@ commFailureAttemptLimit = 5
 #
 doubleClickTimeout = 0.5
 
+# Allow for the customization of key bindings.
+#
+def overrideKeyBindings(script, keyBindings):
+    return keyBindings
+
 # Which packages to search, and the order in which to search,
 # for application settings.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
