@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.50 2006/10/22 20:51:44 wwalker Exp $"
-__version__   = "$Revision: 1.50 $"
-__date__      = "$Date: 2006/10/22 20:51:44 $"
+__id__        = "$Id: util.py,v 1.55 2006/12/15 18:25:11 lmonsanto Exp $"
+__version__   = "$Revision: 1.55 $"
+__date__      = "$Date: 2006/12/15 18:25:11 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -41,6 +41,7 @@ import atspi
 import chnames
 import debug
 import input_event
+import orca_prefs
 import orca_state
 import phonnames
 import punctuation_settings
@@ -289,7 +290,7 @@ def __getDisplayedTextInComboBox(combo):
             textObj = child
 
     if textObj:
-        [displayedText, startOffset, endOffset] = getTextLineAtCaret(textObj)
+        [displayedText, caretOffset, startOffset] = getTextLineAtCaret(textObj)
         #print "TEXTOBJ", displayedText
     else:
         selectedItem = None
@@ -312,7 +313,8 @@ def __getDisplayedTextInComboBox(combo):
             displayedText = combo.name
             #print "NAME", displayedText
         elif combo.text:
-            [displayedText, startOffset, endOffset] = getTextLineAtCaret(combo)
+            [displayedText, caretOffset, startOffset] = \
+                getTextLineAtCaret(combo)
             #print "TEXT", displayedText
 
     return displayedText
@@ -759,19 +761,29 @@ def getTopLevel(obj):
                   + obj.accessibleNameToString())
 
     while obj \
+          and obj.parent \
           and (obj != obj.parent) \
           and (obj.parent.role != rolenames.ROLE_APPLICATION):
         obj = obj.parent
         debug.println(debug.LEVEL_FINEST, "--> obj.name="
                       + obj.accessibleNameToString())
 
-    if obj and (obj.parent.role == rolenames.ROLE_APPLICATION):
+    if obj and obj.parent and (obj.parent.role == rolenames.ROLE_APPLICATION):
         pass
     else:
         obj = None
 
     return obj
 
+def getTopLevelName(obj):
+    """ Returns the name of the top-level object. See getTopLevel.
+    """
+    top = getTopLevel(obj)
+    if (not top) or (not top.name):
+        return ""
+    else:
+        return top.name
+
 def getTextLineAtCaret(obj):
     """Gets the line of text where the caret is.
 
@@ -779,7 +791,8 @@ def getTextLineAtCaret(obj):
     - obj: an Accessible object that implements the AccessibleText
            interface
 
-    Returns the line of text where the caret is.
+    Returns the [string, caretOffset, startOffset] for the line of text
+    where the caret is.
     """
 
     # Get the the AccessibleText interrface
@@ -788,21 +801,23 @@ def getTextLineAtCaret(obj):
     if not text:
         return ["", 0, 0]
 
+    if text.caretOffset == text.characterCount:
+        caretOffset = max(0, text.caretOffset - 1)
+    else:
+        caretOffset = text.caretOffset
+
     # Get the line containing the caret
     #
-    offset = text.caretOffset
-    line = text.getTextAtOffset(offset,
-                                atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+    [string, startOffset, endOffset] = text.getTextAtOffset(
+        caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
 
-    # Line is actually a list of objects-- the first is the actual
-    # text of the line, the second is the start offset, and the third
-    # is the end offset.  Sometimes we get the trailing line-feed-- remove it
+    # Sometimes we get the trailing line-feed-- remove it
     #
-    content = line[0].decode("UTF-8")
+    content = string.decode("UTF-8")
     if content[-1:] == "\n":
         content = content[:-1]
 
-    return [content.encode("UTF-8"), offset, line[1]]
+    return [content.encode("UTF-8"), text.caretOffset, startOffset]
 
 def getNodeLevel(obj):
     """Determines the node level of this object if it is in a tree
@@ -1188,6 +1203,23 @@ def findActiveWindow():
 
     return window
 
+def saveOldAppSettings():
+    """Save a copy of all the existing application specific settings
+    (as specified by the settings.userCustomizableSettings dictionary)."""
+
+    return orca_prefs.readPreferences()
+
+def restoreOldAppSettings(prefsDict):
+    """Restore a copy of all the previous saved application settings.
+
+    Arguments:
+    - prefsDict: the dictionary containing the old application settings.
+    """
+
+    for key in settings.userCustomizableSettings:
+        if prefsDict.has_key(key):
+            settings.__dict__[key] = prefsDict[key]
+
 ########################################################################
 #                                                                      #
 # METHODS FOR DRAWING RECTANGLES AROUND OBJECTS ON THE SCREEN          #
